VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "Mandelbrot"
   ClientHeight    =   4155
   ClientLeft      =   165
   ClientTop       =   855
   ClientWidth     =   4485
   LinkTopic       =   "Form1"
   ScaleHeight     =   4155
   ScaleWidth      =   4485
   StartUpPosition =   3  'Windows Default
   Begin VB.PictureBox picCanvas 
      Height          =   3735
      Left            =   120
      ScaleHeight     =   3675
      ScaleWidth      =   3915
      TabIndex        =   0
      Top             =   120
      Width           =   3975
   End
   Begin VB.Menu mnuFile 
      Caption         =   "&File"
      Begin VB.Menu mnuFileSaveAs 
         Caption         =   "Save &As..."
      End
   End
   Begin VB.Menu mnuScale 
      Caption         =   "&Scale"
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&2"
         Index           =   2
      End
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&4"
         Index           =   4
      End
      Begin VB.Menu mnuScaleFactor 
         Caption         =   "x&8"
         Index           =   8
      End
      Begin VB.Menu mnuScaleFull 
         Caption         =   "&Full"
      End
      Begin VB.Menu mnuScaleSep 
         Caption         =   "-"
      End
      Begin VB.Menu mnuScaleRefresh 
         Caption         =   "&Refresh"
         Shortcut        =   {F5}
      End
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private m_DrawingBox As Boolean
Private m_StartX As Single
Private m_StartY As Single
Private m_CurX As Single
Private m_CurY As Single
Private m_Xmin As Double
Private m_Xmax As Double
Private m_Ymin As Double
Private m_Ymax As Double

Public MaxIterations As Integer
Public C As Complex

Private Const MIN_X As Double = -2
Private Const MAX_X As Double = 2
Private Const MIN_Y As Double = -2
Private Const MAX_Y As Double = 2

Private Sub Form_Load()
    Me.Show
    DoEvents

    picCanvas.AutoRedraw = True
    picCanvas.ScaleMode = vbPixels

    MaxIterations = 64
    Set C = New Complex
    C.Initialize 0.62, -0.55

    ' Display the first Vortex set.
    mnuScaleFull_Click
End Sub

Private Sub picCanvas_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    m_DrawingBox = True
    m_StartX = X
    m_StartY = Y
    m_CurX = X
    m_CurY = Y
End Sub

Private Sub picCanvas_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    If Not m_DrawingBox Then Exit Sub

    m_CurX = X
    m_CurY = Y
    picCanvas.Cls
    picCanvas.Line (m_StartX, m_StartY)-(m_CurX, m_CurY), vbYellow, B
End Sub

Private Sub picCanvas_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim x1 As Double
Dim x2 As Double
Dim y1 As Double
Dim y2 As Double
Dim factor As Double
    
    If Not m_DrawingBox Then Exit Sub
    m_DrawingBox = False
    picCanvas.Cls

    m_CurX = X
    m_CurY = Y

    ' Put the coordinates in proper order.
    x1 = Min(m_StartX, m_CurX)
    x2 = Max(m_StartX, m_CurX)
    If x1 = x2 Then x2 = x1 + 1

    y1 = Min(m_StartY, m_CurY)
    y2 = Max(m_StartY, m_CurY)
    If y1 = y2 Then y2 = y1 + 1

    ' Convert screen coords into drawing coords.
    factor = (m_Xmax - m_Xmin) / picCanvas.ScaleWidth
    m_Xmax = m_Xmin + x2 * factor
    m_Xmin = m_Xmin + x1 * factor

    factor = (m_Ymax - m_Ymin) / picCanvas.ScaleHeight
    m_Ymax = m_Ymin + y2 * factor
    m_Ymin = m_Ymin + y1 * factor

    Me.MousePointer = vbHourglass
    DoEvents
    DrawVortex
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

Private Sub Form_Resize()
    picCanvas.Move 0, 0, ScaleWidth, ScaleHeight
End Sub

Private Sub mnuFileSaveAs_Click()
    MsgBox "Not yet implemented."
End Sub

Private Sub mnuScaleFactor_Click(Index As Integer)
    ScaleArea Index
End Sub

' Adjust the aspect ratio of the selected
' coordinates so they fit the window properly.
Private Sub AdjustAspect()
Dim want_aspect As Double
Dim picCanvas_aspect As Double
Dim hgt As Double
Dim wid As Double
Dim mid As Double

    want_aspect = (m_Ymax - m_Ymin) / (m_Xmax - m_Xmin)
    picCanvas_aspect = picCanvas.ScaleHeight / picCanvas.ScaleWidth
    If want_aspect > picCanvas_aspect Then
        ' The selected area is too tall and thin.
        ' Make it wider.
        wid = (m_Ymax - m_Ymin) / picCanvas_aspect
        mid = (m_Xmin + m_Xmax) / 2
        m_Xmin = mid - wid / 2
        m_Xmax = mid + wid / 2
    Else
        ' The selected area is too short and wide.
        ' Make it taller.
        hgt = (m_Xmax - m_Xmin) * picCanvas_aspect
        mid = (m_Ymin + m_Ymax) / 2
        m_Ymin = mid - hgt / 2
        m_Ymax = mid + hgt / 2
    End If
End Sub

' Draw the Vortex set.
Private Sub DrawVortex()
' Work until the magnitude squared > 4.
Const MAX_MAG_SQUARED As Integer = 4

Dim wid As Integer
Dim hgt As Integer
Dim clr As Integer
Dim X As Integer
Dim Y As Integer
Dim dReaC As Double
Dim dImaC As Double
Dim Zrea As Double
Dim Zima As Double
Dim ZN As Complex
Dim ZNminus1 As Complex
Dim ZNplus1 As Complex
Dim Z1 As Complex
Dim Z2 As Complex

    ' Clear.
    picCanvas.Line (0, 0)-(picCanvas.ScaleWidth, picCanvas.ScaleHeight), vbBlack, BF
    DoEvents

    ' Adjust the coordinate bounds to fit picCanvas.
    AdjustAspect

    ' dReaC is the change in the real part
    ' (X value) for C. dImaC is the change in the
    ' imaginary part (Y value).
    wid = picCanvas.ScaleWidth
    hgt = picCanvas.ScaleHeight
    dReaC = (m_Xmax - m_Xmin) / (wid - 1)
    dImaC = (m_Ymax - m_Ymin) / (hgt - 1)

    ' Calculate the values.
    Set Z1 = New Complex
    Set Z2 = New Complex
    Zrea = m_Xmin
    For X = 0 To wid - 1
        Zima = m_Ymin
        For Y = 0 To hgt - 1
            Set ZN = New Complex
            ZN.Initialize Zrea, Zima
            Set ZNminus1 = New Complex
            ZNminus1.Initialize Zrea, Zima
                    
            clr = 1
            Do While clr < MaxIterations And ZN.MagnitudeSquared < MAX_MAG_SQUARED
                ' Calculate Z(clr).
                Z1.Initialize C.Real, 0
                Z2.Initialize C.Imaginary, 0
                Set ZNplus1 = ZN.Times(ZN).Plus(Z1).Plus(Z2.Times(ZNminus1))
                Set ZNminus1 = ZN
                Set ZN = ZNplus1
                clr = clr + 1
            Loop

            ' Set the pixel's value.
            picCanvas.PSet (X, Y), QBColor(clr Mod 16)

            Zima = Zima + dImaC
        Next Y
        Zrea = Zrea + dReaC

        ' Let the user know we're not dead.
        If X Mod 10 = 0 Then
            picCanvas.Refresh
        End If
    Next X

    Me.Caption = "Vortex (" & _
        m_Xmin & ", " & m_Ymin & ")-(" & _
        m_Xmax & ", " & m_Ymax & ")"

    picCanvas.Picture = picCanvas.Image
End Sub

' Scale the selected area by this factor.
Private Sub ScaleArea(ByVal scale_factor As Integer)
Dim size As Double
Dim mid As Double

    size = scale_factor * (m_Xmax - m_Xmin)
    If size > 3.2 Then
        mnuScaleFull_Click
        Exit Sub
    End If
    mid = (m_Xmin + m_Xmax) / 2
    m_Xmin = mid - size / 2
    m_Xmax = mid + size / 2

    size = scale_factor * (m_Ymax - m_Ymin)
    If size > 2.4 Then
        mnuScaleFull_Click
        Exit Sub
    End If
    mid = (m_Ymin + m_Ymax) / 2
    m_Ymin = mid - size / 2
    m_Ymax = mid + size / 2

    Me.MousePointer = vbHourglass
    DoEvents
    DrawVortex
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

' Zoom out to full scale.
Private Sub mnuScaleFull_Click()
    m_Xmin = MIN_X
    m_Xmax = MAX_X
    m_Ymin = MIN_Y
    m_Ymax = MAX_Y

    Me.MousePointer = vbHourglass
    DoEvents
    DrawVortex
    Me.MousePointer = vbDefault
    picCanvas.MousePointer = vbCrosshair
End Sub

' Refresh. This is useful when the user resizes the form.
Private Sub mnuScaleRefresh_Click()
    ScaleArea 1
End Sub

Private Function Min(ByVal v1 As Double, ByVal v2 As Double) As Double
    If v1 < v2 Then
        Min = v1
    Else
        Min = v2
    End If
End Function
Private Function Max(ByVal v1 As Double, ByVal v2 As Double) As Double
    If v1 < v2 Then
        Max = v2
    Else
        Max = v1
    End If
End Function
